#include <pt.h>
#include <Wire.h>
#include <LCD.h>
#include <LiquidCrystal_I2C.h>

//กำหนด ขา pin
const int SCLK_pin=PIN_PC0;
const int RCLK_pin=PIN_PC1;
const int DIO_pin =PIN_PC2;

#define I2C_ADDR 0x27
#define BACKLIGHT_PIN 3
#define RESET_SW PIN_PB2
#define UP_SW PIN_PB3
#define DOWN_SW PIN_PB4
#define LED_Red PIN_PB1
#define LED_Green PIN_PB0

#define PT_DELAY(pt, ms, ts) \
    ts = millis(); \
    PT_WAIT_WHILE(pt, millis()-ts < (ms));
    
int  disp[8];
unsigned long prev =0;
unsigned long waitMS=0;

LiquidCrystal_I2C lcd(I2C_ADDR,2,1,0,4,5,6,7);
char disp_c[8] ;

struct pt pt_amount;
struct pt pt_NumShow;
struct pt pt_LCDShow;
struct pt pt_Score;
struct pt pt_Reset;
struct pt pt_Timer;

int n;
unsigned long start=millis();
byte b = 0;
int preAmount;
int postAmount = 10;
int num1 = 0;
int num10 = 0;
int score = 0;
int sta = 0;
int limittime = 20;

/////////////////////////////////////////////////////// Amount
PT_THREAD(amount(struct pt* pt))
{
  static uint32_t ts;
 
  PT_BEGIN(pt);
 
  while (1)
  {
    PT_WAIT_UNTIL(pt, digitalRead(UP_SW) == LOW);
    PT_DELAY(pt, 10, ts);

    PT_WAIT_UNTIL(pt, digitalRead(UP_SW) == HIGH);
    PT_DELAY(pt, 10, ts);
    
    postAmount = random(1, 100);
    if (postAmount == preAmount) {
      postAmount = random(1, 100);
    }
    num1 = postAmount%10;
    num10 = (postAmount-(num1))/10;
    //Serial.println(postAmount);
    //Serial.println(num1);
    //Serial.println(num10);
  }
 
  PT_END(pt);
}

/////////////////////////////////////////////////////// Num-Show
PT_THREAD(NumShow(struct pt* pt))
{
  static uint32_t ts;
 
  PT_BEGIN(pt);
 
  while (1)
  {
    PT_DELAY(pt, 0.0000001, ts);
    showDisplay();
    if ( millis() > (prev + waitMS))
    {
      //code to loop in here

      // แสดงผลแบบตัวอักษร ตัวอย่างแสดงคำว่า Ardu
      // char a = '23';
      // showText(' ',' ',a,'1'); //  แก้ค่าที่จะให้แสดงผลตรงนี้

      // แสดงผลแบบตัวเลข ตัวอย่างแสดงค่า 95.42
         showText(0,0,num10,num1); //15 --> แสดงเลข 5.
      
      // หน่วงเวลา
      //wait(1000);
    }
  }
 
  PT_END(pt);
}

/////////////////////////////////////////////////////// LCDShow
PT_THREAD(LCDShow(struct pt* pt))
{
  static uint32_t ts;
 
  PT_BEGIN(pt);

  lcd.clear();
 
  while (1)
  {
    PT_DELAY(pt, 0.0000001, ts);
    lcd.setCursor(0,0);
    lcd.print("Score : "); 
    lcd.setCursor(8,0);
    lcd.print(score); 
    lcd.setCursor(0,1);
    lcd.print("Time : "); 
    if (limittime < 10) {
      lcd.setCursor(7,1);
      lcd.print(limittime);
      lcd.setCursor(8,1);
      lcd.print(" ");
    }
    else if (limittime > 9) {
      lcd.setCursor(7,1);
      lcd.print(limittime);
    }
  }
 
  PT_END(pt);
}

/////////////////////////////////////////////////////// Timer
PT_THREAD(Timer(struct pt* pt))
{
  static uint32_t ts;
 
  PT_BEGIN(pt);
 
  while (1)
  {
    PT_DELAY(pt, 1000, ts);
    if (limittime >0) {
      limittime = limittime-1;
    }
    if (limittime == 0) {
      lcd.clear();
      lcd.setCursor(3,0);
      lcd.print("Time's Up !");
      lcd.setCursor(0,1);
      lcd.print("Your Score is ");
      lcd.setCursor(14,1);
      lcd.print(score);
      while (digitalRead(RESET_SW) == HIGH) {
        delay(100);
      }
    }
  }
 
  PT_END(pt);
}

/////////////////////////////////////////////////////// Score
PT_THREAD(Score(struct pt* pt))
{
  static uint32_t ts;
 
  PT_BEGIN(pt);
 
  while (digitalRead(UP_SW) == LOW || digitalRead(DOWN_SW) == LOW)
  {
    PT_DELAY(pt, 0.0000001, ts);
    if (preAmount < postAmount) {
      if (digitalRead(UP_SW) == LOW) {
        digitalWrite(LED_Red, LOW);
        digitalWrite(LED_Green, HIGH);
        score = score+1;
        preAmount = postAmount;
    
        postAmount = random(1, 100);
        if (postAmount == preAmount) {
          postAmount = random(1, 100);
        }
        num1 = postAmount%10;
        num10 = (postAmount-(num1))/10;
        PT_DELAY(pt, 200, ts);
      }
      else {
        digitalWrite(LED_Green, LOW);
        digitalWrite(LED_Red, HIGH);
      }
    }
    else if (preAmount > postAmount) {
      if (digitalRead(DOWN_SW) == LOW) {
        digitalWrite(LED_Red, LOW);
        digitalWrite(LED_Green, HIGH);
        score = score+1;
        preAmount = postAmount;

        postAmount = random(1, 100);
        if (postAmount == preAmount) {
          postAmount = random(1, 100);
        }
        num1 = postAmount%10;
        num10 = (postAmount-(num1))/10;
        PT_DELAY(pt, 200, ts);
      }
      else {
        digitalWrite(LED_Green, LOW);
        digitalWrite(LED_Red, HIGH);
      }
    }
  }
 
  PT_END(pt);
}

/////////////////////////////////////////////////////// Reset
PT_THREAD(Reset(struct pt* pt))
{
  static uint32_t ts;
 
  PT_BEGIN(pt);
 
  while (1)
  {
    PT_DELAY(pt, 0.0000001, ts);
    preAmount = 0;
    postAmount = 10;
    num1 = 0;
    num10 = 0;
    score = 0;
    sta = 0;
    limittime = 20;
  }
 
  PT_END(pt);
}

/////////////////////////////////////////////////////// Main

void setup() {
  pinMode(RESET_SW, INPUT_PULLUP);
  pinMode(UP_SW, INPUT_PULLUP);
  pinMode(DOWN_SW, INPUT_PULLUP);
  pinMode(RCLK_pin,OUTPUT);
  pinMode(DIO_pin,OUTPUT);
  pinMode(SCLK_pin,OUTPUT);
  pinMode(LED_Red,OUTPUT);
  pinMode(LED_Green,OUTPUT);

  lcd.begin (16,2);  
  // Switch on the backlight
  lcd.setBacklightPin(BACKLIGHT_PIN,POSITIVE);
  lcd.setBacklight(HIGH);
  lcd.home (); // ไปที่ตัวอักษรที่ 0 แถวที่ 1
  
  Serial.begin(9600);
  lcd.clear();
  lcd.setCursor(3,0);
  lcd.print("WELCOME to"); 
  lcd.setCursor(2,1);
  lcd.print("HIGH or LOW"); 
  delay(5000);
  lcd.clear();
  lcd.setCursor(2,0);
  lcd.print("PRESS RESET"); 
  lcd.setCursor(3,1);
  lcd.print("TO START !"); 
  while(digitalRead(RESET_SW) == HIGH) {
    delay(50);
  }
}

void loop() {
  while (digitalRead(RESET_SW) == HIGH) {
    Score(&pt_Score);
    NumShow(&pt_NumShow);
    LCDShow(&pt_LCDShow);
    Timer(&pt_Timer);
  }
  lcd.clear();
  Reset(&pt_Reset);
}

/////////////////////////////////////////////////////// 7-Segment
void showText(char a, char b , char c,char d) {
  disp_c[0] = d;
  disp_c[1] = c;
  disp_c[2] = b;
  disp_c[3] = a;
}
void showDisplay() {
  setDisp();
  for(int i=0; i<8; i++) {
    setDigit(i,disp[i]);
  }
}

void setDigit(int dig, int character) {
  int digits[]= {128,64,32,16,8,4,2,1};

  //character set (0-9)0-9
  //            (10-19)0.-9.
  //            (20-45)A-Z
  //            (46-71)a-z
  //            (72)- (73) space
  int characters[]= {3,159,37,13,153,73,65,31,1,9,2,158,36,12,152,72,64,30,0,8,17,1,99,3,97,113,67,145,243,135,145,227,85,19,3,49,25,115,73,31,129,129,169,145,137,37,5,193,229,133,33,113,9,209,247,143,81,227,85,213,197,49,25,245,73,225,199,199,169,145,137,37,253,255};

  digitalWrite(RCLK_pin, LOW);
  shiftOut(DIO_pin, SCLK_pin, LSBFIRST, characters[character]);
  shiftOut(DIO_pin, SCLK_pin, LSBFIRST, digits[dig]);
  digitalWrite(RCLK_pin, HIGH);
}

void setDisp() {
  for (int i=0; i<8;i++) {
    int val = disp_c[i];
    if((val >= 32)&&(val <= 47)) {        // ! ถึง / ไม่ให้แสดง
      switch (val){
      case 45 :
        val = 72;
        break;  // เครื่องหมาย -
      default :
        val = 73;
        break;  // ค่าอื่นระหว่างนี้ ไม่ให้แสดง
      }
    }
    else if((val >= 48)&&(val <= 57)) {   //0-9
      val -= 48;
    }
    else if((val >= 65)&&(val <= 90)) {   //A-Z
      val -= 45;
    }
    else if((val >= 97)&&(val <= 122)) {  //a-z
      val -= 51;
    }

    disp[i] = val;

  }
}   

void wait( unsigned long milsec)
{
  prev = millis();
  waitMS = milsec;
}
///////////////////////////////////////////////////////
